
/**
 * RmiJdbc client/server JDBC Driver
 * (C) GIE Dyade (Groupe BULL / INRIA Research Center) 1997
 *
 * @version     1.0
 * @author      Pierre-Yves Gibello (Pierre-Yves.Gibello@inrialpes.fr)
 */

package RmiJdbc;

import java.sql.*;

import java.rmi.*;
import java.rmi.server.Unreferenced;

// TBD: WARNING This file contains a hack for InputStream class...
// InputStream is not serializable, of course !
// The right way would be to encapsulate InputStream in a RMI remote object
// (hope I'll find time to do that)

/**
 * <P>A SQL statement is pre-compiled and stored in a
 * PreparedStatement object. This object can then be used to
 * efficiently execute this statement multiple times. 
 *
 * <P><B>Note:</B> The setXXX methods for setting IN parameter values
 * must specify types that are compatible with the defined SQL type of
 * the input parameter. For instance, if the IN parameter has SQL type
 * Integer then setInt should be used.
 *
 * <p>If arbitrary parameter type conversions are required then the
 * setObject method should be used with a target SQL type.
 *
 * @see Connection#prepareStatement
 * @see ResultSet 
 */

public class RJPreparedStatementServer
extends RJStatementServer
implements RJPreparedStatementInterface, Unreferenced {

  java.sql.PreparedStatement jdbcPrepStmt_;

  public RJPreparedStatementServer(java.sql.PreparedStatement p)
  throws RemoteException {
    super(p);
    jdbcPrepStmt_ = p;
  }

  public void unreferenced() { Runtime.getRuntime().gc(); }

  /**
   * A prepared SQL query is executed and its ResultSet is returned.
   *
   * @return a ResultSet that contains the data produced by the
   * query; never null
   */
  public RJResultSetInterface executeQuery()
  throws RemoteException, SQLException {
    return new RJResultSetServer(jdbcPrepStmt_.executeQuery());
  }

    /**
     * Execute a SQL INSERT, UPDATE or DELETE statement. In addition,
     * SQL statements that return nothing such as SQL DDL statements
     * can be executed.
     *
     * @return either the row count for INSERT, UPDATE or DELETE; or 0
     * for SQL statements that return nothing
     */
  public int executeUpdate() throws RemoteException, SQLException {
    return jdbcPrepStmt_.executeUpdate();
  }

    /**
     * Set a parameter to SQL NULL.
     *
     * <P><B>Note:</B> You must specify the parameter's SQL type.
     *
     * @param parameterIndex the first parameter is 1, the second is 2, ...
     * @param sqlType SQL type code defined by java.sql.Types
     */
  public void setNull(int parameterIndex, int sqlType)
  throws RemoteException, SQLException {
      jdbcPrepStmt_.setNull(parameterIndex, sqlType);
  }

    /**
     * Set a parameter to a Java boolean value.  The driver converts this
     * to a SQL BIT value when it sends it to the database.
     *
     * @param parameterIndex the first parameter is 1, the second is 2, ...
     * @param x the parameter value
     */
  public void setBoolean(int parameterIndex, boolean x)
  throws RemoteException, SQLException {
    jdbcPrepStmt_.setBoolean(parameterIndex, x);
  }

    /**
     * Set a parameter to a Java byte value.  The driver converts this
     * to a SQL TINYINT value when it sends it to the database.
     *
     * @param parameterIndex the first parameter is 1, the second is 2, ...
     * @param x the parameter value
     */
  public void setByte(int parameterIndex, byte x)
  throws RemoteException, SQLException {
    jdbcPrepStmt_.setByte(parameterIndex, x);
  }

    /**
     * Set a parameter to a Java short value.  The driver converts this
     * to a SQL SMALLINT value when it sends it to the database.
     *
     * @param parameterIndex the first parameter is 1, the second is 2, ...
     * @param x the parameter value
     */
  public void setShort(int parameterIndex, short x)
  throws RemoteException, SQLException {
    jdbcPrepStmt_.setShort(parameterIndex, x);
  }

    /**
     * Set a parameter to a Java int value.  The driver converts this
     * to a SQL INTEGER value when it sends it to the database.
     *
     * @param parameterIndex the first parameter is 1, the second is 2, ...
     * @param x the parameter value
     */
  public void setInt(int parameterIndex, int x)
  throws RemoteException, SQLException {
    jdbcPrepStmt_.setInt(parameterIndex, x);
  }

    /**
     * Set a parameter to a Java long value.  The driver converts this
     * to a SQL BIGINT value when it sends it to the database.
     *
     * @param parameterIndex the first parameter is 1, the second is 2, ...
     * @param x the parameter value
     */
  public void setLong(int parameterIndex, long x)
  throws RemoteException, SQLException {
    jdbcPrepStmt_.setLong(parameterIndex, x);
  }

    /**
     * Set a parameter to a Java float value.  The driver converts this
     * to a SQL FLOAT value when it sends it to the database.
     *
     * @param parameterIndex the first parameter is 1, the second is 2, ...
     * @param x the parameter value
     */
  public void setFloat(int parameterIndex, float x) 
  throws RemoteException, SQLException {
    jdbcPrepStmt_.setFloat(parameterIndex, x);
  }

    /**
     * Set a parameter to a Java double value.  The driver converts this
     * to a SQL DOUBLE value when it sends it to the database.
     *
     * @param parameterIndex the first parameter is 1, the second is 2, ...
     * @param x the parameter value
     */
  public void setDouble(int parameterIndex, double x)
  throws RemoteException, SQLException {
    jdbcPrepStmt_.setDouble(parameterIndex, x);
  }

    /**
     * Set a parameter to a java.lang.BigDecimal value.  The driver converts
     * this to a SQL NUMERIC value when it sends it to the database.
     *
     * @param parameterIndex the first parameter is 1, the second is 2, ...
     * @param x the parameter value
     */
  public void setBigDecimal(int parameterIndex, java.math.BigDecimal x)
  throws RemoteException, SQLException {
    jdbcPrepStmt_.setBigDecimal(parameterIndex, x);
  }

    /**
     * Set a parameter to a Java String value.  The driver converts this
     * to a SQL VARCHAR or LONGVARCHAR value (depending on the arguments
     * size relative to the driver's limits on VARCHARs) when it sends
     * it to the database.
     *
     * @param parameterIndex the first parameter is 1, the second is 2, ...
     * @param x the parameter value
     */
  public void setString(int parameterIndex, String x)
  throws RemoteException, SQLException {
    jdbcPrepStmt_.setString(parameterIndex, x);
  }

    /**
     * Set a parameter to a Java array of bytes.  The driver converts
     * this to a SQL VARBINARY or LONGVARBINARY (depending on the
     * argument's size relative to the driver's limits on VARBINARYs)
     * when it sends it to the database.
     *
     * @param parameterIndex the first parameter is 1, the second is 2, ...
     * @param x the parameter value 
     */
  public void setBytes(int parameterIndex, byte x[])
  throws RemoteException, SQLException {
    jdbcPrepStmt_.setBytes(parameterIndex, x);
  }

    /**
     * Set a parameter to a java.sql.Date value.  The driver converts this
     * to a SQL DATE value when it sends it to the database.
     *
     * @param parameterIndex the first parameter is 1, the second is 2, ...
     * @param x the parameter value
     */
  public void setDate(int parameterIndex, java.sql.Date x)
  throws RemoteException, SQLException {
    jdbcPrepStmt_.setDate(parameterIndex, x);
  }

    /**
     * Set a parameter to a java.sql.Time value.  The driver converts this
     * to a SQL TIME value when it sends it to the database.
     *
     * @param parameterIndex the first parameter is 1, the second is 2, ...
     * @param x the parameter value
     */
  public void setTime(int parameterIndex, java.sql.Time x) 
  throws RemoteException, SQLException {
    jdbcPrepStmt_.setTime(parameterIndex, x);
  }

    /**
     * Set a parameter to a java.sql.Timestamp value.  The driver
     * converts this to a SQL TIMESTAMP value when it sends it to the
     * database.
     *
     * @param parameterIndex the first parameter is 1, the second is 2, ...
     * @param x the parameter value 
     */
  public void setTimestamp(int parameterIndex, java.sql.Timestamp x)
  throws RemoteException, SQLException {
    jdbcPrepStmt_.setTimestamp(parameterIndex, x);
  }

    /**
     * When a very large ASCII value is input to a LONGVARCHAR
     * parameter, it may be more practical to send it via a
     * java.io.InputStream. JDBC will read the data from the stream
     * as needed, until it reaches end-of-file.  The JDBC driver will
     * do any necessary conversion from ASCII to the database char format.
     * 
     * <P><B>Note:</B> This stream object can either be a standard
     * Java stream object or your own subclass that implements the
     * standard interface.
     *
     * @param parameterIndex the first parameter is 1, the second is 2, ...
     * @param x the java input stream which contains the ASCII parameter value
     * @param length the number of bytes in the stream 
     */
//TBD This is a hack (InputStream not serializable)
//  public void setAsciiStream(int parameterIndex, java.io.InputStream x,
  public void setAsciiStream(int parameterIndex, byte[] x,
  int length) throws RemoteException, SQLException {
//    jdbcPrepStmt_.setAsciiStream(parameterIndex, x, length);
    jdbcPrepStmt_.setAsciiStream(parameterIndex,
      new java.io.ByteArrayInputStream(x), length);
  }

    /**
     * When a very large UNICODE value is input to a LONGVARCHAR
     * parameter, it may be more practical to send it via a
     * java.io.InputStream. JDBC will read the data from the stream
     * as needed, until it reaches end-of-file.  The JDBC driver will
     * do any necessary conversion from UNICODE to the database char format.
     * 
     * <P><B>Note:</B> This stream object can either be a standard
     * Java stream object or your own subclass that implements the
     * standard interface.
     *
     * @param parameterIndex the first parameter is 1, the second is 2, ...  
     * @param x the java input stream which contains the
     * UNICODE parameter value 
     * @param length the number of bytes in the stream 
     */
//TBD This is a hack (InputStream not serializable)
//  public void setUnicodeStream(int parameterIndex, java.io.InputStream x,
  public void setUnicodeStream(int parameterIndex, byte[] x,
  int length) throws RemoteException, SQLException {
//    jdbcPrepStmt_.setUnicodeStream(parameterIndex, x, length);
    jdbcPrepStmt_.setUnicodeStream(parameterIndex,
      new java.io.ByteArrayInputStream(x), length);
  }

    /**
     * When a very large binary value is input to a LONGVARBINARY
     * parameter, it may be more practical to send it via a
     * java.io.InputStream. JDBC will read the data from the stream
     * as needed, until it reaches end-of-file.
     * 
     * <P><B>Note:</B> This stream object can either be a standard
     * Java stream object or your own subclass that implements the
     * standard interface.
     *
     * @param parameterIndex the first parameter is 1, the second is 2, ...
     * @param x the java input stream which contains the binary parameter value
     * @param length the number of bytes in the stream 
     */
//TBD This is a hack (InputStream not serializable)
//  public void setBinaryStream(int parameterIndex, java.io.InputStream x,
  public void setBinaryStream(int parameterIndex, byte[] x,
  int length) throws RemoteException, SQLException {
//    jdbcPrepStmt_.setBinaryStream(parameterIndex, x, length);
    jdbcPrepStmt_.setBinaryStream(parameterIndex,
      new java.io.ByteArrayInputStream(x), length);
  }

    /**
     * <P>In general, parameter values remain in force for repeated use of a
     * Statement. Setting a parameter value automatically clears its
     * previous value.  However, in some cases it is useful to immediately
     * release the resources used by the current parameter values; this can
     * be done by calling clearParameters.
     */
  public void clearParameters() throws RemoteException, SQLException {
    jdbcPrepStmt_.clearParameters();
  }

    //----------------------------------------------------------------------
    // Advanced features:

    /**
     * <p>Set the value of a parameter using an object; use the
     * java.lang equivalent objects for integral values.
     *
     * <p>The given Java object will be converted to the targetSqlType
     * before being sent to the database.
     *
     * <p>Note that this method may be used to pass datatabase-
     * specific abstract data types. This is done by using a Driver-
     * specific Java type and using a targetSqlType of
     * java.sql.types.OTHER.
     *
     * @param parameterIndex The first parameter is 1, the second is 2, ...
     * @param x The object containing the input parameter value
     * @param targetSqlType The SQL type (as defined in java.sql.Types) to be 
     * sent to the database. The scale argument may further qualify this type.
     * @param scale For java.sql.Types.DECIMAL or java.sql.Types.NUMERIC types
     *          this is the number of digits after the decimal.  For all other
     *          types this value will be ignored,
     * @see Types 
     */
  public void setObject(int parameterIndex, Object x, int targetSqlType,
  int scale) throws RemoteException, SQLException {
    jdbcPrepStmt_.setObject(parameterIndex, x, targetSqlType, scale);
  }

    /**
      * This method is like setObject above, but assumes a scale of zero.
      */
  public void setObject(int parameterIndex, Object x, int targetSqlType)
  throws RemoteException, SQLException {
    jdbcPrepStmt_.setObject(parameterIndex, x, targetSqlType);
  }

    /**
     * <p>Set the value of a parameter using an object; use the
     * java.lang equivalent objects for integral values.
     *
     * <p>The JDBC specification specifies a standard mapping from
     * Java Object types to SQL types.  The given argument java object
     * will be converted to the corresponding SQL type before being
     * sent to the database.
     *
     * <p>Note that this method may be used to pass datatabase
     * specific abstract data types, by using a Driver specific Java
     * type.
     *
     * @param parameterIndex The first parameter is 1, the second is 2, ...
     * @param x The object containing the input parameter value 
     */
  public void setObject(int parameterIndex, Object x)
  throws RemoteException, SQLException {
    jdbcPrepStmt_.setObject(parameterIndex, x);
  }

    /**
     * Some prepared statements return multiple results; the execute
     * method handles these complex statements as well as the simpler
     * form of statements handled by executeQuery and executeUpdate.
     *
     * @see Statement#execute
     */
  public boolean execute() throws RemoteException, SQLException {
    return jdbcPrepStmt_.execute();
  }

};

